@@ -27,7 +27,7 @@ import re
 
 # Local
 from base.g import *
-from base import utils, tui
+from base import utils, tui, password, os_utils, smart_install
 from prnt import cups
 from installer import core_install
 
@@ -82,39 +82,29 @@ def check_user_groups():
                 result = True
                 break
             cnt += 1
-            
+
     return result
 
-# This function adds the groups ('_cups') to user 
-def add_group(core):
+# This function adds the groups ('_cups') to user
+def add_group(core, mode, passwordObj):
     result = False
     add_user_to_group = core.get_distro_ver_data('add_user_to_group', '')
     if add_user_to_group:
         usermod = os.path.join(utils.which("usermod"), "usermod") + " %s %s" % (add_user_to_group, prop.username)
     else:
-        usermod = os.path.join(utils.which("usermod"), "usermod") + " %s %s" % ("-Glp", prop.username)
+        usermod = os.path.join(utils.which("usermod"), "usermod") + " %s -a %s" % ("-G _cups", prop.username)
 
-    su_sudo = utils.su_sudo()
-    password_f = None
-    if su_sudo is "su":
-        name,version,is_su = utils.os_release()
-        log.debug("name = %s version = %s is_su = %s" %(name,version,is_su))
-        if ( name == 'Fedora' and version >= '14' and is_su == True):
-           #using su opening GUI apps fail in Fedora 14. 
-           #To run GUI apps as root, you need a root login shell (su -) in Fedora 14   
-           su_sudo = 'su - -c "%s"'
-        else:
-           su_sudo = 'su -c "%s"'
+    if passwordObj == None:
+        passwordObj = password.Password(mode)
 
-        password_f = "get_password_ui"
-#        password_f = utils.get_password
-    cmd =su_sudo % usermod
+    cmd =passwordObj.getAuthCmd() % usermod
     log.info("cmd = %s" %cmd)
-#    sts, output = utils.run(cmd, True, password_f, -1,True,cmd)  
-    sts, output = utils.run(cmd, True, password_f, 1, True, "Please enter root/superuser password to add '_cups' group")
+
+    pswd_msg = "Enter root/superuser password to add '_cups' group"
+    sts, output = utils.run(cmd, passwordObj, pswd_msg)
     if sts == 0:
         result = True
-        
+
     return result
 
 
@@ -131,14 +121,14 @@ def addToDeviceList(Key, printer_name, device_uri,back_end, ppd_fileType,PPDFile
     else:
         log.warn("%s is not HP Device." %(printer_name))
 
-#Validate all the Queues 
-def parseQueues():
+#Validate all the Queues
+def parseQueues(mode):
     is_hpcups_installed = to_bool(sys_conf.get('configure', 'hpcups-install', '0'))
     is_hpijs_installed = to_bool(sys_conf.get('configure', 'hpijs-install', '0'))
     status, output = utils.run('lpstat -v')
 
     cups_printers = []
-    if output.find("No destinations added") != -1:
+    if output.find("No destinations added") != -1 or output.find("lpstat:") != -1:
         log.info("No Queue added")
     else:
         for p in output.splitlines():
@@ -190,11 +180,12 @@ def parseQueues():
                     desc = ''
 
             log.debug("PPD Description: %s" % desc)
-            status, output = utils.run('lpstat -p%s' % printer_name)
+            cmd= 'lpstat -p%s' % printer_name
+            status, output = utils.run(cmd)
             log.debug("Printer status: %s" % output.replace("\n", ""))
 
             #### checking for USb devices ####
-            if USB_PATTERN.search(device_uri):                   
+            if USB_PATTERN.search(device_uri):
                 Key =USB_PATTERN.search(device_uri).group(1)
             #### checking for network devices ####
             elif NET_PATTERN.search(device_uri):
@@ -277,8 +268,8 @@ def reconfigure_Queue(que, mode, dialog= None,app=None):
                 log.info("' %s' removed successfully.\nRe-configuring this printer by hp-setup..."%que.PrinterName)
                 path = utils.which('hp-setup')
                 if path:
-                    log.debug("Starting hp-setup")
-                    os.system('hp-setup -i')
+                    cmd = 'hp-setup -i'
+                    os_utils.execute(cmd)
         elif que.IsEnabled == False:
             Error_Found = True
             responce, value =tui.enter_yes_no("'%s Queue is paused. Do you want to enable queue?"%(que.PrinterName))
@@ -349,7 +340,7 @@ def parseDeviceURI(device_uri):
     return back_end, is_hp, bus, model, serial, dev_file, host, zc, port
 
 
-def main_function(mode = GUI_MODE, ui_toolkit= UI_TOOLKIT_QT4, quiet_mode = False, check_grps= False,DEVICE_URI=None):
+def main_function(passwordObj = None, mode = GUI_MODE, ui_toolkit= UI_TOOLKIT_QT4, quiet_mode = False, check_grps= False,DEVICE_URI=None):
     global Error_Found
     try:
         from base import device, pml
@@ -362,12 +353,12 @@ def main_function(mode = GUI_MODE, ui_toolkit= UI_TOOLKIT_QT4, quiet_mode = Fals
         if check_grps and check_user_groups() is False:
             core = core_install.CoreInstall(core_install.MODE_CHECK)
             core.init()
-            if add_group(core) is False:
+            if add_group(core, mode, passwordObj) is False:
                 Error_Found = True
                 log.error("Failed to add _cups group to user[%s]. Manually add '_cups' group to usergroups. And reboot system."%prop.username)
             else:
                 log.info("Groups added successfully and reboot is required. Please reboot system to take effect.")
-        mapofDevices = parseQueues()
+        mapofDevices = parseQueues(mode)
         if mapofDevices.items() == 0:
             log.debug("No queues found.")
 
@@ -376,14 +367,18 @@ def main_function(mode = GUI_MODE, ui_toolkit= UI_TOOLKIT_QT4, quiet_mode = Fals
                 if not quiet_mode:
                     Error_Found = True
                     log.warn("%d queues of same device %s is configured.\nRemove unwanted queues."%(len(val),val[0].PrinterName))
-                 
+
                 for que in val:
                     reconfigure_Queue(que, mode)
             else:
                 log.debug("")
                 log.debug("Single print queue is configured for '%s'. " %val[0].PrinterName)
                 reconfigure_Queue(val[0], mode)
-            
+
+        SI_sts = smart_install.disable(mode, '', None, None, passwordObj)
+        if SI_sts != smart_install.SIH_NO_SI_DEVICES:
+            Error_Found = True
+
         if Error_Found is False:
             if not quiet_mode:
                 if len(mapofDevices) == 0:
@@ -391,7 +386,7 @@ def main_function(mode = GUI_MODE, ui_toolkit= UI_TOOLKIT_QT4, quiet_mode = Fals
                 else:
 #                    log.info(log.green("Queue(s) configured correctly using HPLIP."))
                     log.info("Queue(s) configured correctly using HPLIP.")
-                
+
     elif mode == GUI_MODE:
         # Only Qt4 is supported.
         if ui_toolkit == 'qt3':
@@ -405,17 +400,17 @@ def main_function(mode = GUI_MODE, ui_toolkit= UI_TOOLKIT_QT4, quiet_mode = Fals
             log.error("Unable to load Qt4 support. Is it installed?")
             sys.exit(1)
         app = QApplication(sys.argv)
-        dialog = QueuesDiagnose(None, "","",QUEUES_MSG_SENDING)
+        dialog = QueuesDiagnose(None, "","",QUEUES_MSG_SENDING,passwordObj)
         if check_grps and check_user_groups() is False:
             core = core_install.CoreInstall(core_install.MODE_CHECK)
             core.init()
-            if add_group(core) is False:
+            if add_group(core, mode, passwordObj) is False:
                 Error_Found = True
                 dialog.showMessage("User must be part of '_cups' group.\nManually add '_cups' group to '%s' user. " %prop.username)
             else:
                 dialog.showSuccessMessage("Groups added successfully and reboot is required. Please reboot system to take effect.")
 
-        mapofDevices = parseQueues()
+        mapofDevices = parseQueues(mode)
         if mapofDevices.items() == 0:
             log.debug("No queues found.")
 
@@ -427,12 +422,16 @@ def main_function(mode = GUI_MODE, ui_toolkit= UI_TOOLKIT_QT4, quiet_mode = Fals
                     dialog.showMessage("%d queues of same device %s is configured.\nRemove unwanted queues."%(len(val),val[0].PrinterName))
                 for que in val:
                     reconfigure_Queue(que, mode,dialog,app)
-                    
+
             else:
                 log.debug("")
                 log.debug("Single print queue is configured for '%s'. " %val[0].PrinterName)
                 reconfigure_Queue(val[0], mode,dialog, app)
-            
+
+        SI_sts = smart_install.disable(mode, ui_toolkit, dialog, app, passwordObj)
+        if SI_sts != smart_install.SIH_NO_SI_DEVICES:
+            Error_Found = True
+
         if Error_Found is False:
             if not quiet_mode:
                 if len(mapofDevices) == 0:
