@@ -16,7 +16,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 #
-# Author: Don Welch
+# Author: Don Welch, Narla Naga Samrat Chowdary, Yashwant Kumar Sahu
 #
 
 from __future__ import division
@@ -24,7 +24,21 @@ from __future__ import division
 # Std Lib
 import struct
 import cStringIO
-import xml.parsers.expat as expat
+from base.g import *
+try:
+   import xml.parsers.expat as expat
+except ImportError,e:
+   log.info("\n")
+   log.error("Failed to import xml.parsers.expat(%s).\nThis may be due to the incompatible version of python-xml package.\n"%(e))
+   if "undefined symbol" in str(e):
+       log.info(log.blue("Please re-install compatible version (other than 2.7.2-7.14.1) due to bug reported at 'https://bugzilla.novell.com/show_bug.cgi?id=766778'."))
+       log.info(log.blue("\n        Run the following commands in root mode to change the python-xml package.(i.e Installing 2.7.2-7.1.2)"))
+       log.info(log.blue("\n        Using zypper:\n        'zypper remove python-xml'\n        'zypper install python-xml-2.7.2-7.1.2'"))
+       log.info(log.blue("\n        Using apt-get:\n        'apt-get remove python-xml'\n        'apt-get install python-xml-2.7.2-7.1.2'"))
+       log.info(log.blue("\n        Using yum:\n        'yum remove python-xml'\n        'yum install python-xml-2.7.2-7.1.2'"))
+
+   sys.exit(1)
+
 import re
 import urllib
 try:
@@ -403,6 +417,9 @@ COLORANT_INDEX_TO_AGENT_TYPE_MAP = {
                                     'magenta':  AGENT_TYPE_MAGENTA,
                                     'yellow' :  AGENT_TYPE_YELLOW,
                                     'black' :   AGENT_TYPE_BLACK,
+                                    'photoblack': AGENT_TYPE_PHOTO_BLACK,
+                                    'matteblack' : AGENT_TYPE_MATTE_BLACK,
+                                    'gray' : AGENT_TYPE_LG,
                                    }
 
 MARKER_SUPPLES_TYPE_TO_AGENT_KIND_MAP = {
@@ -553,6 +570,7 @@ def StatusType3( dev, parsedID ): # LaserJet Status (PML/SNMP)
                     else: # SUCCESS
                         if colorant_value is not None:
                             log.debug("colorant value: %s" % colorant_value)
+                            colorant_value = colorant_value.lower().strip()
                             agent_type = COLORANT_INDEX_TO_AGENT_TYPE_MAP.get( colorant_value, AGENT_TYPE_BLACK )
 
                         if agent_type == AGENT_TYPE_NONE:
@@ -1463,6 +1481,7 @@ element_type10_xlate = { 'ink' : AGENT_KIND_SUPPLY,
                          'inkCartridge' : AGENT_KIND_HEAD_AND_SUPPLY,
                          'printhead' : AGENT_KIND_HEAD,
                          'toner' : AGENT_KIND_TONER_CARTRIDGE,
+                         'tonerCartridge' : AGENT_KIND_TONER_CARTRIDGE,
                        }
 
 pen_type10_xlate = { 'pK' : AGENT_TYPE_PG,
@@ -1485,31 +1504,35 @@ pen_health10_xlate = { 'ok' : AGENT_HEALTH_OK,
                        'missing' : AGENT_HEALTH_MISINSTALLED,
                      }
 
-def StatusType10FetchUrl(dev, url):
-    if dev.is_local:
-       data_fp = cStringIO.StringIO()
-       dev.getEWSUrl_LEDM(url, data_fp)
-       data = data_fp.getvalue()
-
-    else:
-        if dev.zc:
-            status, ip = hpmudext.get_zc_ip_address(dev.zc)
-            if status != hpmudext.HPMUD_R_OK:
-                log.error("unable to get IP address of mDNS configured device")
-                return None
-        else:
-            ip = dev.host
-
-        # Get the agent status XML
-        addr = "http://%s:8080%s" % (ip, url)
-        feed = urllib.urlopen(addr)
-        data = feed.read()
-        feed.close()
 
+#ExtractXMLData will extract actual data from http response (Transfer-encoding:  chunked).
+#For unchunked response it will not do anything.
+def ExtractXMLData(data):
+    if data[0] is not '<':
+        size = -1
+        temp = ""
+        while size:
+            index = data.find('\r\n')
+            size = int(data[0:index+1], 16)
+            temp = temp + data[index+2:index+2+size]
+            data = data[index+2+size+2:len(data)]
+        data = temp
     return data
 
+def StatusType10FetchUrl(func, url, footer=""):
+    data_fp = cStringIO.StringIO()
+    if footer:
+        data = func(url, data_fp, footer)
+    else:
+        data = func(url, data_fp)
 
-def StatusType10(dev): # Low End Data Model
+    if data:
+        data = data.split('\r\n\r\n', 1)[1]
+        if data:
+            data = ExtractXMLData(data)
+    return data
+
+def StatusType10(func): # Low End Data Model
     status_block = { 'revision' :    STATUS_REV_UNKNOWN,
                      'agents' :      [],
                      'top-door' :    TOP_DOOR_NOT_PRESENT,
@@ -1526,12 +1549,27 @@ def StatusType10(dev): # Low End Data Model
         log.error("cannot get status for printer. please load ElementTree module")
         return status_block
 
+    status_block = StatusType10Agents(func)
+
+    temp_status_block = {}
+    temp_status_block = StatusType10Media(func)
+    status_block.update(temp_status_block)
+
+    temp_status_block = {}
+    temp_status_block = StatusType10Status(func)
+    status_block.update(temp_status_block)
+
+    return status_block
+
+
+def StatusType10Agents(func): # Low End Data Model
+    status_block = {}
     # Get the dynamic consumables configuration
-    data = StatusType10FetchUrl(dev, "/DevMgmt/ConsumableConfigDyn.xml")
+    data = StatusType10FetchUrl(func, "/DevMgmt/ConsumableConfigDyn.xml")
     if not data:
         return status_block
     data = data.replace("ccdyn:", "").replace("dd:", "")
-
+   
     # Parse the agent status XML
     agents = []
     try:
@@ -1543,39 +1581,60 @@ def StatusType10(dev): # Low End Data Model
         for e in elements:
             health = AGENT_HEALTH_OK
             ink_level = 0
-            type = e.find("ConsumableTypeEnum").text
-            state = e.find("ConsumableLifeState/ConsumableState").text
+            agent_sku = ''
+            try:
+                type = e.find("ConsumableTypeEnum").text
+                state = e.find("ConsumableLifeState/ConsumableState").text
 
-            # level
-            if type == "ink" or type == "inkCartridge" or type == "toner":
-                ink_type = e.find("ConsumableLabelCode").text
-                if state != "missing":
-                    try:
-                       ink_level = int(e.find("ConsumablePercentageLevelRemaining").text)
-                    except:
-                       ink_level = 0
-            else:
-                ink_type = ''
-                if state == "ok":
-                    ink_level = 100
+                # level
+                if type == "ink" or type == "inkCartridge" or type == "toner" or type == "tonerCartridge":
+                    ink_type = e.find("ConsumableLabelCode").text
+                    if state != "missing":
+                        try:
+                           ink_level = int(e.find("ConsumablePercentageLevelRemaining").text)
+                           if ink_level == 0:
+                               state = "empty"
+                           elif ink_level <=10:
+                               state = "low"
 
-            log.debug("type '%s' state '%s' ink_type '%s' ink_level %d" % (type, state, ink_type, ink_level))
-    
-            entry = { 'kind' : element_type10_xlate.get(type, AGENT_KIND_NONE),
-                      'type' : pen_type10_xlate.get(ink_type, AGENT_TYPE_NONE),
-                      'health' : pen_health10_xlate.get(state, AGENT_HEALTH_OK),
-                      'level' : int(ink_level),
-                      'level-trigger' : pen_level10_xlate.get(state, AGENT_LEVEL_TRIGGER_SUFFICIENT_0)
-                    }
+                           agent_sku = 'Unknown' #Initialize to unknown. IN some old devices, ConsumableSelectibilityNumber is not returned by device.
+                        except:
+                           ink_level = 0
+                elif type == "printhead":
+                     continue; #No need of adding this agent.
+                else:
+                    ink_type = ''
+                    if state == "ok":
+                        ink_level = 100
 
-            log.debug("%s" % entry)
-            agents.append(entry)
+                try:
+                    agent_sku = e.find("ConsumableSelectibilityNumber").text
+                except:
+                    pass
+
+                log.debug("type '%s' state '%s' ink_type '%s' ink_level %d agent_sku = %s" % (type, state, ink_type, ink_level,agent_sku))
+
+                entry = { 'kind' : element_type10_xlate.get(type, AGENT_KIND_NONE),
+                          'type' : pen_type10_xlate.get(ink_type, AGENT_TYPE_NONE),
+                          'health' : pen_health10_xlate.get(state, AGENT_HEALTH_OK),
+                          'level' : int(ink_level),
+                          'level-trigger' : pen_level10_xlate.get(state, AGENT_LEVEL_TRIGGER_SUFFICIENT_0),
+                          'agent-sku' : agent_sku
+                        }
+
+                log.debug("%s" % entry)
+                agents.append(entry)
+            except AttributeError:
+                log.debug("no value found for attribute")
     except (expat.ExpatError, UnboundLocalError):
         agents = []
     status_block['agents'] = agents
+    return status_block						  
 
+def StatusType10Media(func): # Low End Data Model
+    status_block = {}
     # Get the media handling configuration
-    data = StatusType10FetchUrl(dev, "/DevMgmt/MediaHandlingDyn.xml")
+    data = StatusType10FetchUrl(func, "/DevMgmt/MediaHandlingDyn.xml")
     if not data:
         return status_block
     data = data.replace("mhdyn:", "").replace("dd:", "")
@@ -1608,13 +1667,17 @@ def StatusType10(dev): # Low End Data Model
         if e.text == "autoDuplexor":
             status_block['duplexer'] = DUPLEXER_DOOR_CLOSED
 
+    return status_block
+
+def StatusType10Status(func): # Low End Data Model
+    status_block = {}
     # Get the product status
-    data = StatusType10FetchUrl(dev, "/DevMgmt/ProductStatusDyn.xml")
+    data = StatusType10FetchUrl(func, "/DevMgmt/ProductStatusDyn.xml")
     if not data:
         return status_block
     data = data.replace("psdyn:", "").replace("locid:", "")
     data = data.replace("pscat:", "").replace("dd:", "").replace("ad:", "")
-
+	
     # Parse the product status XML
     try:
         if etree_loaded:
@@ -1624,8 +1687,14 @@ def StatusType10(dev): # Low End Data Model
         elements = tree.findall("Status/StatusCategory")
     except (expat.ExpatError, UnboundLocalError):
         elements = []
+
     for e in elements:
-        if e.text == "closeDoorOrCover":
+
+        if e.text == "processing":
+            status_block['status-code'] = STATUS_PRINTER_PRINTING
+        elif e.text == "ready":
+            status_block['status-code'] = STATUS_PRINTER_IDLE
+        elif e.text == "closeDoorOrCover":
             status_block['status-code'] = STATUS_PRINTER_DOOR_OPEN
         elif e.text == "shuttingDown":
             status_block['status-code'] = STATUS_PRINTER_TURNING_OFF
@@ -1639,7 +1708,7 @@ def StatusType10(dev): # Low End Data Model
             status_block['status-code'] = STATUS_PRINTER_HARD_ERROR
         elif e.text == "outputBinFull":
             status_block['status-code'] = STATUS_PRINTER_OUTPUT_BIN_FULL
-        elif e.text == "unexpectedSizeInTray":
+        elif e.text == "unexpectedSizeInTray" or e.text == "sizeMismatchInTray":
             status_block['status-code'] = STATUS_PRINTER_MEDIA_SIZE_MISMATCH
         elif e.text == "insertOrCloseTray2":
             status_block['status-code'] = STATUS_PRINTER_TRAY_2_MISSING
@@ -1681,5 +1750,58 @@ def StatusType10(dev): # Low End Data Model
             status_block['status-code'] = STATUS_PRINTER_CARTRIDGE_WRONG
         elif e.text == "cartridgeMissing":
             status_block['status-code'] = STATUS_PRINTER_CARTRIDGE_MISSING
+        elif e.text == "missingPrintHead":
+            status_block['status-code'] = STATUS_PRINTER_PRINTHEAD_MISSING
 
-    return status_block
\ No newline at end of file
+
+		#Alert messages for Pentane products RQ 8888
+        elif e.text == "scannerADFMispick":
+            status_block['status-code'] = STATUS_SCANNER_ADF_MISPICK
+
+        elif e.text == "mediaTooShortToAutoDuplex":
+            status_block['status-code'] = STATUS_PRINTER_PAPER_TOO_SHORT_TO_AUTODUPLEX
+
+        elif e.text == "insertOrCloseTray":
+            status_block['status-code'] = STATUS_PRINTER_TRAY_2_3_DOOR_OPEN
+
+        elif e.text == "inkTooLowToPrime":
+            status_block['status-code'] = STATUS_PRINTER_INK_TOO_LOW_TO_PRIME
+
+        elif e.text == "cartridgeVeryLow":
+            status_block['status-code'] = STATUS_PRINTER_VERY_LOW_ON_INK
+
+        elif e.text == "wasteMarkerCollectorAlmostFull":
+            status_block['status-code'] = STATUS_PRINTER_SERVICE_INK_CONTAINER_ALMOST_FULL
+
+        elif e.text == "wasteMarkerCollectorFull":
+            status_block['status-code'] = STATUS_PRINTER_SERVICE_INK_CONTAINER_FULL
+
+        elif e.text == "wasteMarkerCollectorFullPrompt":
+            status_block['status-code'] = STATUS_PRINTER_SERVICE_INK_CONTAINER_FULL_PROMPT
+
+        elif e.text == "missingDuplexer":
+            status_block['status-code'] = STATUS_PRINTER_DUPLEX_MODULE_MISSING
+
+        elif e.text == "printBarStall":
+            status_block['status-code'] = STATUS_PRINTER_PRINTHEAD_JAM
+
+        elif e.text == "outputBinClosed":
+            status_block['status-code'] = STATUS_PRINTER_CLEAR_OUTPUT_AREA
+
+        elif e.text == "outputBinOpened":
+            status_block['status-code'] = STATUS_PRINTER_CLEAR_OUTPUT_AREA
+
+        elif e.text == "reseatDuplexer":
+            status_block['status-code'] = STATUS_PRINTER_RESEAT_DUPLEXER
+
+        elif e.text == "unexpectedTypeInTray":
+            status_block['status-code'] = STATUS_PRINTER_MEDIA_TYPE_MISMATCH
+
+        elif e.text == "manuallyFeed":
+            status_block['status-code'] = STATUS_MANUALLY_FEED
+
+        else:
+            status_block['status-code'] = STATUS_UNKNOWN_CODE
+
+
+    return status_block
