@@ -20,7 +20,7 @@
 # Author: Amarnath Chitumalla
 #
 
-__version__ = '1.0'
+__version__ = '1.1'
 __title__ = 'HP device setup using USB'
 __mod__ = 'hp-config_usb_printer'
 __doc__ = "Detects HP printers connected using USB and installs HPLIP printers and faxes in the CUPS spooler. Tries to automatically determine the correct PPD file to use."
@@ -35,14 +35,16 @@ import time
 
 # Local
 from base.g import *
-from base import device,utils, tui, models,module
+from base import device,utils, tui, models,module, services
 from prnt import cups
-
+from installer import pluginhandler
 
 LPSTAT_PAT = re.compile(r"""(\S*): (.*)""", re.IGNORECASE)
 USB_PATTERN = re.compile(r'''serial=(.*)''',re.IGNORECASE)
 BACK_END_PATTERN = re.compile(r'''(.*):(.*)''',re.IGNORECASE)
+USB_SERIAL_INTERFACE = re.compile(r'''(.*)&interface.*''',re.IGNORECASE)
 DBUS_SERVICE='com.hplip.StatusService'
+DBUS_AVIALABLE=False
 
 ##### METHODS #####
 
@@ -63,6 +65,9 @@ def get_already_added_queues(udev_MDL, udev_serial_no, udev_back_end,remove_non_
 
             back_end = BACK_END_PATTERN.search(device_uri).group(1)
             serial = USB_PATTERN.search(device_uri).group(1)
+            if USB_SERIAL_INTERFACE.search(serial):
+                serial = USB_SERIAL_INTERFACE.search(serial).group(1)
+
             log.debug("udev_serial_no[%s] serial[%s] udev_back_end[%s] back_end[%s]"%(udev_serial_no, serial, udev_back_end, back_end))
             if udev_serial_no == serial and (udev_back_end == back_end or back_end == 'usb'):
                 if remove_non_hp_config and printer_name.find('_') == -1 and printer_name.find('-') != -1:
@@ -83,38 +88,19 @@ def check_cups_process():
     sts, output = utils.run('lpstat -r')
     if sts == 0 and ('is running' in output):
         cups_running_sts = True
-    
+
     return cups_running_sts
 
-
-def showPasswordUI(prompt):
-    import getpass
-    print ""
-    print log.bold(prompt)
-    username = raw_input("Username: ")
-    password = getpass.getpass("Password: ")
-
-    return (username, password)
-
-
-# Restart cups
-def restart_cups():
-    if os.path.exists('/etc/init.d/cups'):
-        return '/etc/init.d/cups restart'
-
-    elif os.path.exists('/etc/init.d/cupsys'):
-        return '/etc/init.d/cupsys restart'
-
-    else:
-        return 'killall -HUP cupsd'
-
-
 # Send dbus event to hpssd on dbus system bus
 def send_message(device_uri, printer_name, event_code, username, job_id, title, pipe_name=''):
+    if DBUS_AVIALABLE == False:
+        return
+
     log.debug("send_message() entered")
     args = [device_uri, printer_name, event_code, username, job_id, title, pipe_name]
     msg = lowlevel.SignalMessage('/', DBUS_SERVICE, 'Event')
     msg.append(signature='ssisiss', *args)
+
     SystemBus().send_message(msg)
     log.debug("send_message() returning")
 
@@ -126,31 +112,27 @@ def usage(typ='text'):
 
 # Systray service. If hp-systray is not running, starts.
 def start_systray():
+    if DBUS_AVIALABLE == False:
+        return False
+
     Systray_Is_Running=False
     status,output = utils.Is_Process_Running('hp-systray')
     if status is False:
-        log.debug("hp-systray is not running.")
         if os.getuid() == 0:
-            log.error("Run \'hp-systray &\' in a terminal. ")
+            log.error(" hp-systray must be running.\n Run \'hp-systray &\' in a terminal. ")
         else:
-            log.debug("Starting hp-systray service")
-            child_pid = os.fork()
-            if child_pid == 0:
-                status,output =utils.run('hp-systray &', True, None, 1, False)
-                if status is not 0:
-                    log.error("Failed to start \'hp-systray\' service. Manually run \'hp-sysray &\' from terminal as non-root user.")
-                sys.exit()
-            else:
-                time.sleep(1)
-                status,output = utils.Is_Process_Running('hp-systray')
-                if  status is True:
-                    Systray_Is_Running=True
-    else:
+            log.info("Starting hp-systray service")
+            services.run_systray()
+            status,output = utils.Is_Process_Running('hp-systray')
+
+    if status == True:
         Systray_Is_Running=True
         log.debug("hp-systray service is running\n")
+
     return Systray_Is_Running
 
 
+
 USAGE = [ (__doc__, "", "name", True),
           ("Usage: %s [OPTIONS] [SERIAL NO.|USB bus:device]" % __mod__, "", "summary", True),
           utils.USAGE_OPTIONS,
@@ -173,35 +155,36 @@ USAGE = [ (__doc__, "", "name", True),
         ]
 
 
+mod = module.Module(__mod__, __title__, __version__, __doc__, USAGE, (INTERACTIVE_MODE,), None, run_as_root_ok=True, quiet=True)
+opts, device_uri, printer_name, mode, ui_toolkit, loc = mod.parseStdOpts('gh',['time-out=', 'timeout='],handle_device_printer=False)
 
-mod = module.Module(__mod__, __title__, __version__, __doc__, USAGE, (INTERACTIVE_MODE, GUI_MODE), (UI_TOOLKIT_QT3, UI_TOOLKIT_QT4), run_as_root_ok=True, quiet=True)
-
-opts, device_uri, printer_name, mode, ui_toolkit, loc = \
-    mod.parseStdOpts('gh',['time-out=', 'timeout='],handle_device_printer=False)
-
-
-LOG_FILE = "/var/log/hp/hplip_config_usb_printer.log" 
+LOG_FILE = "/var/log/hp/hplip_config_usb_printer.log"
 if os.path.exists(LOG_FILE):
-    os.remove(LOG_FILE)
+    try:
+        os.remove(LOG_FILE)
+    except OSError:
+        pass
 
 log.set_logfile(LOG_FILE)
 log.set_where(log.LOG_TO_CONSOLE_AND_FILE)
 cmd="chmod 664 "+LOG_FILE
 sts,output = utils.run(cmd)
 if sts != 0:
-    log.warn("Failed to change log file permissions: %s" %output)
+    log.debug("Failed to change log file permissions: %s" %output)
 
 cmd="chgrp lp "+LOG_FILE
 sts,output = utils.run(cmd)
 if sts != 0:
-    log.warn("Failed to change log file group permissions: %s" %output)
+    log.debug("Failed to change log file group permissions: %s" %output)
 
 try:
     import dbus
     from dbus import SystemBus, lowlevel
 except ImportError:
-        log.error("hp-check-plugin Tool requires dBus and python-dbus")
-        sys.exit(1)
+    log.warn("Failed to Import DBUS ")
+    DBUS_AVIALABLE = False
+else:
+    DBUS_AVIALABLE = True
 
 try:
     param = mod.args[0]
@@ -214,15 +197,6 @@ if len(param) < 1:
     sys.exit()
 
 try:
-    #*****************************CHECK SMART INSTALL
-    try:
-        import hpmudext
-    except:
-        log.error("Failed to import hpmudext")
-    else:
-        hpmudext.handle_smartinstall()
-        
-
     # ******************************* MAKEURI
     if param:
         device_uri, sane_uri, fax_uri = device.makeURI(param)
@@ -230,7 +204,7 @@ try:
         log.error("This is not a valid device")
         sys.exit(0)
 
-    # ******************************* QUERY MODEL AND COLLECT PPDS
+    # ******************************* QUERY MODEL AND CHECKING SUPPORT
     log.debug("\nSetting up device: %s\n" % device_uri)
     back_end, is_hp, bus, model, serial, dev_file, host, zc, port = device.parseDeviceURI(device_uri)
 
@@ -238,45 +212,74 @@ try:
     if not mq or mq.get('support-type', SUPPORT_TYPE_NONE) == SUPPORT_TYPE_NONE:
         log.error("Unsupported printer model.")
         sys.exit(1)
+
+    printer_name = ""
+    username = prop.username
+    job_id = 0
+    # ******************************* STARTING CUPS SERVICE, IF NOT RUNNING.
     while check_cups_process() is False:
-	log.debug("CUPS is not running.. waiting for 30 sec")
+        log.debug("CUPS is not running.. waiting for 30 sec")
         time.sleep(30)
 
+    # ******************************* RUNNING HP-SETUP, IF QUEUE IS NOT ADDED
     time.sleep(1)
     norm_model = models.normalizeModelName(model).lower()
     remove_non_hp_config =True
     if not mq.get('fax-type', FAX_TYPE_NONE) in (FAX_TYPE_NONE, FAX_TYPE_NOT_SUPPORTED):
         fax_config_list = get_already_added_queues(norm_model, serial, 'hpfax',remove_non_hp_config)
 
-
     printer_config_list = get_already_added_queues(norm_model, serial, back_end, remove_non_hp_config)
-    if len(printer_config_list) ==0  or len(printer_config_list) == 0:
-        cmd ="hp-setup -i -x -a -q %s"%param
-        log.debug("%s"%cmd)
-        utils.run(cmd)
+    if len(printer_config_list) ==0:
+        if "SMART_INSTALL_ENABLED" not in device_uri:
+            cmd ="hp-setup -i -x -a -q %s"%param
+            log.debug("%s"%cmd)
+            utils.run(cmd)
 
         if start_systray():
-            printer_name = ""
-            username = ""
-            send_message( device_uri, printer_name, EVENT_ADD_PRINTQUEUE, username, 0,'')
-    else:
-        if start_systray():
-            printer_name = ""
-            username = ""
-            send_message( device_uri, printer_name, EVENT_DIAGNOSE_PRINTQUEUE, username, 0,'')
+            if "SMART_INSTALL_ENABLED" in device_uri:
+                send_message( device_uri, printer_name, EVENT_DIAGNOSE_PRINTQUEUE, username, job_id,'')
+            else:
+                send_message( device_uri, printer_name, EVENT_ADD_PRINTQUEUE, username, job_id,'')
 
-    # Cleaning CUPS created Queues. If any,
+    # ******************************* TRIGGERING PLUGIN POP-UP FOR PLUGING SUPPORTED PRINTER'S
+    plugin = mq.get('plugin', PLUGIN_NONE)
+    if plugin != PLUGIN_NONE:
+       pluginObj = pluginhandler.PluginHandle()
+       plugin_sts = pluginObj.getStatus()
+       if plugin_sts == pluginhandler.PLUGIN_INSTALLED:
+          log.info("Device Plugin is already installed")
+       elif plugin_sts == pluginhandler.PLUGIN_NOT_INSTALLED :
+          log.info("HP Device Plug-in is not found")
+       else:
+          log.info("HP Device Plug-in version mismatch or some files are corrupted")
+    
+       if plugin_sts != pluginhandler.PLUGIN_INSTALLED:
+           if start_systray():
+               send_message( device_uri,  printer_name, EVENT_AUTO_CONFIGURE, username, job_id, "AutoConfig")
+
+       # ******************************* RUNNING FIRMWARE DOWNLOAD TO DEVICE FOR SUPPORTED PRINTER'S
+       fw_download_req = mq.get('fw-download', False)
+       if fw_download_req:
+           fw_cmd = utils.which('hp-firmware', True)
+           if fw_cmd:
+                fw_cmd += " -y3 -s %s"%param
+                log.debug(fw_cmd)
+                fw_sts, fw_out = utils.run(fw_cmd)
+                if fw_sts == 0:
+                    log.debug("Firmware downloaded to %s "%device_uri)
+                else:
+                    log.debug("Failed to download firmware to %s device"%device_uri)     
+
+    # ******************************* REMOVING CUPS CREATED QUEUE, If any
     i =0
-    while i <24:
-        time.sleep(5)
-          
+    while i <12:
+        time.sleep(2)
         get_already_added_queues(norm_model, serial, 'hpfax',remove_non_hp_config)
         get_already_added_queues(norm_model, serial, 'hp',remove_non_hp_config)
         if i == 0:
-            send_message( device_uri, printer_name, EVENT_DIAGNOSE_PRINTQUEUE, username, 0,'')
+            send_message( device_uri, printer_name, EVENT_DIAGNOSE_PRINTQUEUE, username, job_id,"")
         i += 1
 
-
 except KeyboardInterrupt:
     log.error("User exit")
 
